// Copyright 2020 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package middleware

import (
	"cp-organization-management-gateway/errors"
	"fmt"
	"github.com/dgrijalva/jwt-go"
	"github.com/gin-gonic/gin"
	"github.com/jaryhe/gopkgs/cache"
	"github.com/jaryhe/gopkgs/jwtwrapper"
	"github.com/tidwall/gjson"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
	"net/http"
	"strings"
	"time"
)

// 响应数据
type Response struct {
	Code    codes.Code `json:"code" default:"1"`
	Message string     `json:"message" default:"success"`
}

func hasPermission(uri string, method string, subject []byte) bool {
	array := strings.Split(uri, "?")
	uri = strings.TrimRight(array[0], "/")
	length := len(uri)
	for i := length; i >= 0; i-- {
		if uri[i] == '/' {
			if uri[i+1] >= '0' && uri[i+1] <= '9'{
				uri = uri[:i]
			}
			break
		}
	}
	permissionKey := fmt.Sprintf("%s-%s", uri, method)
	permissionKey = strings.TrimPrefix(permissionKey, "/api/v1")
	permissions := gjson.GetBytes(subject, "permissions").Array()
	hasPermission := false
	for _, v := range permissions {
		if v.String() == permissionKey {
			hasPermission = true
		}
	}
	if !hasPermission {
		return false
	}
	return true
}

// JWT is jwt middleware
func Jwt() gin.HandlerFunc {
	return func(c *gin.Context) {
		s := status.New(1, "内部服务错误")
		token := c.GetHeader("token")
		if token == "" {
			if v, ok := status.FromError(errors.NoTokenError); ok {
				s = v
			}
			c.JSON(http.StatusOK, Response{s.Code(), s.Message()})
			c.Abort()
			return
		}

		// 解析token
		claims, err := jwtwrapper.ParseToken(token)
		if err != nil {
			switch err.(*jwt.ValidationError).Errors {
			case jwt.ValidationErrorExpired:
				if v, ok := status.FromError(errors.TokenExpiredError); ok {
					s = v
				}
			default:
				if v, ok := status.FromError(errors.TokenFailedError); ok {
					s = v
				}
			}

			c.JSON(http.StatusOK, Response{s.Code(), s.Message()})
			c.Abort()
			return
		}

		// 将claims信息保存到上下文，为后续使用
		c.Set("claims", claims)
		

		// 进行下一次处理
		if gjson.GetBytes([]byte(claims.Subject), "tmp_token").Bool() {
			c.JSON(http.StatusOK, Response{10008, "无效凭据"})
			c.Abort()
			return
		}
/*
		if !hasPermission(c.Request.RequestURI, c.Request.Method, []byte(claims.Subject)) {
			c.JSON(http.StatusOK, Response{10011, "权限不足"})
			c.Abort()
			return
		}
*/
		endTime := gjson.GetBytes([]byte(claims.Subject), "end_time").Int()

		// 机构过期，从redis再看看
		if endTime <= time.Now().Unix() {
			c.JSON(http.StatusOK, Response{10008, "机构已过有效期"})
			c.Abort()
			return
		}
		code := gjson.GetBytes([]byte(claims.Subject), "organization_code").String()
		isEnableStr, _ := cache.Redis().Get("organization_active"+code)
		if isEnableStr == "0" {
			c.JSON(http.StatusOK, Response{10008, "机构已禁用"})
			c.Abort()
			return
		}
		c.Next()
	}
}
